/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Chemm.debug.cpp,v 1.8 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_M -2
#define MAX_M 8
#define MIN_N -2
#define MAX_N 8
#define MIN_LDA -2
#define MAX_LDA 8
#define MIN_LDB -2
#define MAX_LDB 8
#define MIN_LDC -2
#define MAX_LDC 8
#define MAX_ITER 3

void
Chemm_test3(const char *side, const char *uplo, COMPLEX alpha,
    COMPLEX beta)
{
    int errorflag = FALSE;
    for (int n = MIN_N; n < MAX_N; n++) {
	for (int m = MIN_M; m < MAX_M; m++) {
	    int minlda;
	    if (Mlsame(side, "L"))
		minlda = max(1, m);
	    else
		minlda = max(1, n);
	    for (int lda = minlda; lda < MAX_LDA; lda++) {
		for (int ldb = max(1, m); ldb < MAX_LDB; ldb++) {
		    for (int ldc = max(1, m); ldc < MAX_LDC; ldc++) {
#if defined VERBOSE_TEST
			printf
			    ("#n is %d, m is %d, lda is %d, ldb is %d, ldc is %d ",
			    n, m, lda, ldb, ldc);
			printf("side is %s, uplo is %s \n", side, uplo);
#endif
			COMPLEX *A;
			COMPLEX *B;
			COMPLEX *C;
			complex < double >*Ad;
			complex < double >*Bd;
			complex < double >*Cd;

			int ka;

			if (Mlsame(side, "L"))
			    ka = m;
			else
			    ka = n;

			A = new COMPLEX[matlen(lda, ka)];
			B = new COMPLEX[matlen(ldb, n)];
			C = new COMPLEX[matlen(ldc, n)];
			Ad = new complex < double >[matlen(lda, ka)];
			Bd = new complex < double >[matlen(ldb, n)];
			Cd = new complex < double >[matlen(ldc, n)];

			complex < double >alphad, betad;

			alphad.real() = cast2double(alpha.re);
			alphad.imag() = cast2double(alpha.im);

			betad.real() = cast2double(beta.re);
			betad.imag() = cast2double(beta.im);

			for (int iter = 0; iter < MAX_ITER; iter++) {
			    set_random_vector(A, Ad, matlen(lda, ka));
			    set_random_vector(B, Bd, matlen(ldb, n));
			    set_random_vector(C, Cd, matlen(ldc, n));

		            mpack_errno = 0; blas_errno = 0;
			    Chemm(side, uplo, m, n, alpha, A, lda, B, ldb,
				beta, C, ldc);
			    zhemm_f77(side, uplo, &m, &n, &alphad, Ad, &lda,
				Bd, &ldb, &betad, Cd, &ldc);
#if defined VERBOSE_TEST
			    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			     if (mpack_errno != blas_errno) {
			       printf("error in Mxerbla!!\n");
			       exit(1);
			     }

			    double diff = infnorm(C, Cd, matlen(ldc, n), 1);
			    if (fabs(diff) > EPSILON) {
				printf("#error %.16e!!\n", diff);
				errorflag = TRUE;
			    }
			}
			delete[]Cd;
			delete[]Bd;
			delete[]Ad;

			delete[]C;
			delete[]B;
			delete[]A;
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Chemm test failed...\n");
	exit(1);
    }
}

void
Chemm_test2(const char *side, const char *uplo)
{
    COMPLEX alpha, beta;
    COMPLEX Czero, Cone;

    Czero.re = 0.0;
    Czero.im = 0.0;
    Cone.re = 1.0;
    Cone.im = 0.0;

//alpha=*, beta=*
    alpha = mpc_randomnumber();
    beta = mpc_randomnumber();
    Chemm_test3(side, uplo, alpha, beta);

//a=0, b=0;
    alpha = Czero;
    beta = Czero;
    Chemm_test3(side, uplo, alpha, beta);

//a=1, b=0;
    alpha = Cone;
    beta = Czero;
    Chemm_test3(side, uplo, alpha, beta);

//a=0, b=1;
    alpha = Czero;
    beta = Cone;
    Chemm_test3(side, uplo, alpha, beta);

//a=1, b=1;
    alpha = Cone;
    beta = Cone;
    Chemm_test3(side, uplo, alpha, beta);

//a=*, b=0;
    alpha = mpc_randomnumber();
    beta = Czero;
    Chemm_test3(side, uplo, alpha, beta);

//a=*, b=1;
    alpha = mpc_randomnumber();
    beta = Cone;
    Chemm_test3(side, uplo, alpha, beta);

//a=0, b=*;
    alpha = Czero;
    beta = mpc_randomnumber();
    Chemm_test3(side, uplo, alpha, beta);

//a=1, b=*;
    alpha = Cone;
    beta = mpc_randomnumber();
    Chemm_test3(side, uplo, alpha, beta);
}

void
Chemm_test()
{
    Chemm_test2("L", "U");
    Chemm_test2("L", "L");
    Chemm_test2("R", "U");
    Chemm_test2("R", "L");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Chemm_test();
    printf("Chemm test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
