/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Chemv.debug.cpp,v 1.8 2009/09/16 00:41:44 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N -2
#define MAX_N 20
#define MIN_LDA 2
#define MAX_LDA 20
#define MIN_INCX -2
#define MAX_INCX 2
#define MIN_INCY -2
#define MAX_INCY 2
#define MAX_ITER 10

void
Chemv_test3(const char *uplo, COMPLEX alpha, COMPLEX beta)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	    for (int n = MIN_N; n < MAX_N; n++) {
		for (int lda = max(1, n); lda < MAX_LDA; lda++) {
#if defined VERBOSE_TEST
                          printf("#n is %d, lda is %d incx %d incy %d uplo %s\n", n, lda, incx, incy, uplo);
#endif
		    COMPLEX *A = new COMPLEX[matlen(lda, n)];
		    COMPLEX *x = new COMPLEX[veclen(n, incx)];
		    COMPLEX *y = new COMPLEX[veclen(n, incy)];

		    complex < double >*Ad =
			new complex < double >[matlen(lda, n)];
		    complex < double >*xd =
			new complex < double >[veclen(n, incx)];
		    complex < double >*yd =
			new complex < double >[veclen(n, incy)];

		    complex < double >alphad, betad;

		    alphad.real() = cast2double(alpha.re);
		    alphad.imag() = cast2double(alpha.im);

		    betad.real() = cast2double(beta.re);
		    betad.imag() = cast2double(beta.im);

		    for (int iter = 0; iter < MAX_ITER; iter++) {
			set_random_vector(A, Ad, matlen(lda, n));
			set_random_vector(x, xd, veclen(n, incx));
			set_random_vector(y, yd, veclen(n, incy));

	                mpack_errno = 0; blas_errno = 0;
			Chemv(uplo, n, alpha, A, lda, x, incx, beta, y, incy);
			zhemv_f77(uplo, &n, &alphad, Ad, &lda, xd, &incx,
			    &betad, yd, &incy);

#if defined VERBOSE_TEST
			    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			     if (mpack_errno != blas_errno) {
			       printf("error in Mxerbla!!\n");
			       exit(1);
			     }

			double diff = infnorm(y, yd, veclen(n, incy), 1);
			if (fabs(diff) > EPSILON) {
			    printf("#error %.16e!!\n", diff);
			    errorflag = TRUE;
			}
		    }
		    delete[]Ad;
		    delete[]xd;
		    delete[]yd;
		    delete[]x;
		    delete[]y;
		    delete[]A;
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Chemv test failed...\n");
	exit(1);
    }
}

void
Chemv_test2(const char *uplo)
{
    COMPLEX alpha, beta;
    COMPLEX Czero, Cone;

    Czero.re = 0.0;
    Czero.im = 0.0;
    Cone.re = 1.0;
    Cone.im = 0.0;

//alpha=*, beta=*
    alpha = mpc_randomnumber();
    beta = mpc_randomnumber();
    Chemv_test3(uplo, alpha, beta);

//a=0, b=0;
    alpha = Czero;
    beta = Czero;
    Chemv_test3(uplo, alpha, beta);

//a=1, b=0;
    alpha = Cone;
    beta = Czero;
    Chemv_test3(uplo, alpha, beta);

//a=0, b=1;
    alpha = Czero;
    beta = Cone;
    Chemv_test3(uplo, alpha, beta);

//a=1, b=1;
    alpha = Cone;
    beta = Cone;
    Chemv_test3(uplo, alpha, beta);

//a=*, b=0;
    alpha = mpc_randomnumber();
    beta = Czero;
    Chemv_test3(uplo, alpha, beta);

//a=*, b=1;
    alpha = mpc_randomnumber();
    beta = Cone;
    Chemv_test3(uplo, alpha, beta);

//a=0, b=*;
    alpha = Czero;
    beta = mpc_randomnumber();
    Chemv_test3(uplo, alpha, beta);

//a=1, b=*;
    alpha = Cone;
    beta = mpc_randomnumber();
    Chemv_test3(uplo, alpha, beta);
}

void
Chemv_test()
{
    Chemv_test2("U");
    Chemv_test2("L");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Chemv_test();
    printf("Chemv test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
