/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 * 
 * Copyright 2008 by Nakata, Maho
 * 
 * $Id: Rsbmv.debug.cpp,v 1.6 2009/09/16 01:05:09 nakatamaho Exp $ 
 *
 * MPACK - multiple precision arithmetic library
 *
 * This file is part of MPACK.
 *
 * MPACK is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * MPACK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with MPACK.  If not, see
 * <http://www.gnu.org/licenses/lgpl.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/
#include <mblas.h>
#include <blas.h>
#include <mpack_debug.h>

#if defined VERBOSE_TEST
#include <iostream>
#endif

#define MIN_N -1
#define MAX_N 10
#define MAX_K 10
#define MIN_LDA 1
#define MAX_LDA 10
#define MIN_INCX -2
#define MAX_INCX 3
#define MIN_INCY -2
#define MAX_INCY 2
#define MAX_ITER 3

void
Rsbmv_test3(const char *uplo, REAL alpha, REAL beta)
{
    int errorflag = FALSE;

    for (int incx = MIN_INCX; incx <= MAX_INCX; incx++) {
	for (int incy = MIN_INCY; incy < MAX_INCY; incy++) {
	    for (int n = MIN_N; n < MAX_N; n++) {
		for (int k = 0; k < MAX_K; k++) {
		    for (int lda = k + 1; lda < MAX_LDA; lda++) {
#if defined VERBOSE_TEST
			printf("#n is %d, lda is %d, incx is %d, incy is %d ",
			    n, lda, incx, incy);
			printf("uplo is %s \n", uplo);
#endif
			REAL *A = new REAL[matlen(lda, n)];
			REAL *x = new REAL[veclen(n, incx)];
			REAL *y = new REAL[veclen(n, incy)];

			double *Ad = new double[matlen(lda, n)];
			double *xd = new double[veclen(n, incx)];
			double *yd = new double[veclen(n, incy)];

			double alphad, betad;

			alphad = cast2double(alpha);
			betad = cast2double(beta);

			for (int iter = 0; iter < MAX_ITER; iter++) {
			    set_random_vector(A, Ad, matlen(lda, n));
			    set_random_vector(x, xd, veclen(n, incx));
			    set_random_vector(y, yd, veclen(n, incy));

		            mpack_errno = 0; blas_errno = 0;
			    Rsbmv(uplo, n, k, alpha, A, lda, x, incx, beta, y,
				incy);
			    dsbmv_f77(uplo, &n, &k, &alphad, Ad, &lda, xd,
				&incx, &betad, yd, &incy);
#if defined VERBOSE_TEST
			    printf("errno: mpack %d, blas %d\n", mpack_errno, blas_errno);
#endif
			     if (mpack_errno != blas_errno) {
			       printf("error in Mxerbla!!\n");
			       exit(1);
			     }
			    double diff;
			    diff = infnorm(y, yd, veclen(n, incy), 1);
			    if (diff > EPSILON) {
				printf("#error %.16e!!\n", diff);
				errorflag = TRUE;
			    }
			}
			delete[]Ad;
			delete[]xd;
			delete[]yd;
			delete[]x;
			delete[]y;
			delete[]A;
		    }
		}
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Rsbmv test failed...\n");
	exit(1);
    }
}

void
Rsbmv_test2(const char *uplo)
{
    REAL alpha, beta;
    REAL Rzero, Rone;

    Rzero = 0.0;
    Rone = 1.0;

//alpha=*, beta=*
    alpha = mpf_randomnumber();
    beta = mpf_randomnumber();
    Rsbmv_test3(uplo, alpha, beta);

//a=0, b=0;
    alpha = Rzero;
    beta = Rzero;
    Rsbmv_test3(uplo, alpha, beta);

//a=1, b=0;
    alpha = Rone;
    beta = Rzero;
    Rsbmv_test3(uplo, alpha, beta);

//a=0, b=1;
    alpha = Rzero;
    beta = Rone;
    Rsbmv_test3(uplo, alpha, beta);

//a=1, b=1;
    alpha = Rone;
    beta = Rone;
    Rsbmv_test3(uplo, alpha, beta);

//a=*, b=0;
    alpha = mpf_randomnumber();
    beta = Rzero;
    Rsbmv_test3(uplo, alpha, beta);

//a=*, b=1;
    alpha = mpf_randomnumber();
    beta = Rone;
    Rsbmv_test3(uplo, alpha, beta);

//a=0, b=*;
    alpha = Rzero;
    beta = mpf_randomnumber();
    Rsbmv_test3(uplo, alpha, beta);

//a=1, b=*;
    alpha = Rone;
    beta = mpf_randomnumber();
    Rsbmv_test3(uplo, alpha, beta);
}

void
Rsbmv_test()
{
    Rsbmv_test2("U");
    Rsbmv_test2("L");
}

int
main(int argc, char *argv[])
{
#if defined ___MPACK_BUILD_WITH_GMP___
    int default_prec = 256;
    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
#endif
#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
unsigned int oldcw;
fpu_fix_start(&oldcw);
#endif

    Rsbmv_test();
    printf("Rsbmv test passed...\n");

#if defined ___MPACK_BUILD_WITH_QD___ || defined ___MPACK_BUILD_WITH_DD___
fpu_fix_end(&oldcw);
#endif

  return(0);
}
